@@ -1,10 +1,9 @@
 /* Nessus
- * Copyright (C) 1998 Renaud Deraison
+ * Copyright (C) 1998 - 2001 Renaud Deraison
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2,
+ * as published by the Free Software Foundation
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,6 +14,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
+ * In addition, as a special exception, Renaud Deraison
+ * gives permission to link the code of this program with any
+ * version of the OpenSSL library which is distributed under a
+ * license identical to that listed in the included COPYING.OpenSSL
+ * file, and distribute linked combinations including the two.
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * this file, you may extend this exception to your version of the
+ * file, but you are not obligated to do so.  If you do not wish to
+ * do so, delete this exception statement from your version.
+ *
  * This is the Authentication Manager
  *
  */
@@ -27,9 +37,6 @@
 #include "sighand.h"
 #include "globals.h"
 #include "password_dialog.h"
-#ifdef ENABLE_CRYPTO_LAYER
-#include <peks/peks.h>
-#endif
 
 /* 
  * auth_login
@@ -57,18 +64,7 @@ int auth_login(user,password)
 {
   char * buf = emalloc(255);
 
-#ifdef ENABLE_CRYPTO_LAYER
-  if (password == 0)
-    {
-     if(stored_pwd)password = stored_pwd;
-#ifdef USE_GTK
-     else password = (char *)pass_dialog(0);
-#else
-     else password = (char*)cmdline_pass(0);
-#endif
-    }
-#endif
-
+  /* Note: even if we use SSLv3 authentication, we ask for a password anyway */
   network_gets(buf, 7);
   if(strncmp(buf, "User : ", strlen(buf)))return(-1);
   network_printf("%s\n", user);
@@ -86,28 +82,26 @@ int auth_login(user,password)
  * network_printf(
  * 
  * This function sends a string to the server.
+ * In the future, it will have to encrypt the string
+ * but I have not implemented this feature right now
  */
 void network_printf(char * data, ...)
 {
   va_list param;
-  char * buffer = emalloc(8192);
-  int len;
   int r, s;
-  int n = 0;
+  char * buffer = emalloc(8192);
+  int len, n = 0;
   signal(SIGPIPE, sighand_pipe);
   va_start(param, data);
-#ifdef HAVE_VSNPRINTF
+
   s = 8192;
   for(;;)
   {
    r = vsnprintf(buffer, s, data, param);
    if(r >= 0 && r < s)break;
-   s = r >  s ? r + 1:s*2;
+   s = r > s ? r + 1 : s * 2;
    buffer = realloc(buffer, s);
   }
-#else
-  vsprintf(buffer, data,param);
-#endif
   len = strlen(buffer);
   while(n < len)
   {
@@ -118,9 +112,11 @@ void network_printf(char * data, ...)
    {
    int e;
    if((len - m - n) < size)size = len - m - n;
-   e = send(GlobalSocket, &(buffer[n+m]), size, 0);
-   if(e < 0)
+   e = nsend(GlobalSocket, &(buffer[n+m]), size, 0);
+   if(e < 0) {
+     perror("send");
     return;
+   }
    m+=e;
   }
   n+=m;
@@ -134,14 +130,27 @@ void network_printf(char * data, ...)
  * network_gets(
  * 
  * Reads data sent by the server
+ * RETURN: >0 is the amount of data placed in s
+ *         <0 means there was an error.
  */
-char * network_gets(s, size)
+int network_gets(s, size)
      char * s;
      size_t size;
 {
-  recv_line(GlobalSocket, s, size);
-  if(!strlen(s))return(NULL);
-  else return(s);
+  int n;
+  /* We are assuming that recv_line() will block until it has 
+   * recvieved a full line of data, encountered a hard error, or eof 
+   * (socket close?)
+   * Also, recv_line will return 0 on error.
+   */
+  n = recv_line(GlobalSocket, s, size); 
+  if (n > 0)
+    return n;
+  else
+  {
+    s[0] = '\0';    /* zero the buffer */
+    return -1;
+  }
 }
 
 char * network_gets_raw(s, size)
@@ -150,17 +159,10 @@ char * network_gets_raw(s, size)
 {
   int n = 0, processed ;
 
-#ifdef ENABLE_CRYPTO_LAYER
-  int io_state, m = 1;
-
-  /* allow empty frames for channel admin */
-  io_state = io_ctrl (GlobalSocket, IO_STOPONEMPTY_STATE, &m, 0);
-#endif
-
   /* read up until no more data, or a line terminating character 
      '\0' or '\n' is found */
   for (processed = 0; processed < (int)size; processed ++) {
-    if ((n = recv (GlobalSocket, s + processed, 1, 0)) <= 0) {
+    if ((n = nrecv (GlobalSocket, s + processed, 1, 0)) <= 0) {
       /* on error, the characers read so far might be garbage */
       if (n < 0)
 	processed = 0 ;
@@ -176,12 +178,5 @@ char * network_gets_raw(s, size)
   s [processed] = '\0' ;
   if (!processed) s = 0 ;
 
-#ifdef ENABLE_CRYPTO_LAYER
-  /* restore io state while keeping the error state */
-  if (n < 0) m = errno ;
-  io_ctrl (GlobalSocket, IO_STOPONEMPTY_STATE, &io_state, 0); 
-  errno = (n < 0) ? m : 0 ;
-#endif
-
   return s;
 }
