@@ -1,10 +1,9 @@
 /* Nessus
- * Copyright (C) 1998, 1999, 2000 Renaud Deraison
+ * Copyright (C) 1998 - 2001 Renaud Deraison
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2,
+ * as published by the Free Software Foundation
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,11 +14,22 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
+ * In addition, as a special exception, Renaud Deraison
+ * gives permission to link the code of this program with any
+ * version of the OpenSSL library which is distributed under a
+ * license identical to that listed in the included COPYING.OpenSSL
+ * file, and distribute linked combinations including the two.
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * this file, you may extend this exception to your version of the
+ * file, but you are not obligated to do so.  If you do not wish to
+ * do so, delete this exception statement from your version.
  *
  */
 
 #include <includes.h>
 
+#include "backend.h"
 #if USE_GTK 
 #include <gtk/gtk.h>
 
@@ -32,6 +42,8 @@
 #include "globals.h"
 #include "error_dialog.h"
 #include "xpm/computer.xpm"
+#include "monitor_dialog.h"
+#include "report_ng.h"
 
 /*static char ** plugins_order_table = NULL;*/
 static harglst * plugins_order_table = NULL;
@@ -43,7 +55,44 @@ static void monitor_list_update(struct arglist *, char *, int);
 static int monitor_stop_whole_test_destroy(void*, void*, struct arglist *);
 static int monitor_stop_whole_test(GtkWidget * , struct arglist *);
 static void build_plugins_order_table(char *);
-static char * label_name(char * , GtkWidget * , GtkWidget * );
+
+struct gui_host {
+	GtkWidget * gui;
+	struct arglist * ctrls;
+	struct gui_host * next;
+};
+
+
+
+static int 
+is_server_present(soc)
+	int soc;
+{
+	fd_set  rd;
+	struct timeval tv = {2,0};
+	int fd = nessus_get_socket_from_connection(soc);
+
+	if(fd < 0 || fd >= FD_SETSIZE)
+	{
+	 fprintf(stderr, "is_server_present: fd(%d) out of range\n", fd);
+	 return 0;
+	}
+	FD_ZERO(&rd);
+	FD_SET(fd, &rd);
+	if(select(fd+1, &rd, NULL, NULL, &tv) > 0)
+	{
+		int len = -1;
+		ioctl(fd, FIONREAD, &len);
+		if(!len){
+			return 0;
+			}
+	}
+	return 1;
+}
+
+
+
+
 
 /*
  * Function called when the UI is idle, which checks
@@ -56,13 +105,23 @@ idle_socket(struct arglist * ctrls)
 {
   fd_set rd;
   struct timeval tv = {0,100};
-  int n;
-  if(GlobalSocket < 0)return FALSE;
+  int n, soc;
+  if(GlobalSocket < 0)
+    {
+      fprintf(stderr, "idle_socket: GlobalSocket=%d\n", GlobalSocket);
+      return FALSE;
+    }
+  soc = nessus_get_socket_from_connection(GlobalSocket);
+  if((soc < 0) || (soc >= FD_SETSIZE))
+  {
+   fprintf(stderr, "idle_server: soc(%d) out of range\n", soc);
+   return -1;
+  }
   FD_ZERO(&rd);
-  FD_SET(GlobalSocket, &rd);
-  n = select(GlobalSocket+1, &rd, NULL, NULL, &tv);
-  if(FD_ISSET(GlobalSocket, &rd)&& (n > 0))
-    monitor_input_callback(ctrls, GlobalSocket, 0);
+  FD_SET(soc, &rd);
+  n = select(soc+1, &rd, NULL, NULL, &tv);
+  if(FD_ISSET(soc, &rd)&& (n > 0))
+    monitor_input_callback(ctrls, soc /* ? Not used */, 0);
   return TRUE;
 }
 
@@ -80,19 +139,32 @@ monitor_dialog_setup(victim, restore)
   struct arglist * ctrls = emalloc(sizeof(struct arglist));
   GtkWidget * scrolled_window; 
   GtkWidget * w, * box;
+  char* window_title;
+  char* host_name;
   int tag;
-  struct arglist * hosts;
+  int backend = backend_init(NULL);
 
-  hosts = emalloc(sizeof(struct arglist));
 
-  arg_add_value(ctrls, "HOSTS", ARG_ARGLIST, -1, hosts);
+  arg_add_value(ctrls, "MONITOR_BACKEND", ARG_INT, -1, (void*)backend);
   
   w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+#if GTK_VERSION > 10
   gtk_window_set_default_size(GTK_WINDOW(w), 640,480);
+#else
+  gtk_widget_set_usize(GTK_WIDGET(w), 640, 480);
+#endif
+
   gtk_widget_realize(w);
   gtk_signal_connect(GTK_OBJECT(w), "delete_event",
 		     (GtkSignalFunc)monitor_stop_whole_test_destroy,ctrls);
-  gtk_window_set_title(GTK_WINDOW(w), "Nessus portscanning/attack status");
+		     
+  host_name = arg_get_value(Prefs, "nessusd_host");
+  if(host_name)window_title = emalloc(strlen(host_name) + 255);
+  else window_title = emalloc(255);
+  sprintf(window_title, "Scanning network from %s",
+  	host_name?host_name:"some host");		     
+  gtk_window_set_title(GTK_WINDOW(w), window_title);
+  efree(&window_title);
   gtk_container_border_width(GTK_CONTAINER(w), 10);
   arg_add_value(ctrls, "WINDOW", ARG_PTR, -1, w);
   gtk_widget_show(w);
@@ -159,39 +231,34 @@ monitor_list_update(ctrls,msg, short_status)
  GtkWidget * gtkw;
  char * list_hostname;
  int flag = 0;
+ static harglst * hosts = NULL;
+ int v = 1;
+ 
+ if(!hosts)
+ {
+  hosts = harg_create(65000);
+ }
+
  
  if(!short_status)
- {
-  hostname = emalloc(strlen(msg)+1);
-  action = emalloc(strlen(msg)+1);
-  parse_nessusd_status(msg, hostname, action, &current, &max);
- }
+  parse_nessusd_status(msg, &hostname, &action, &current, &max);
  else
+  parse_nessusd_short_status(msg, &hostname, &action, &current, &max);
+ 
+ if(hostname)
  {
-  char * t;
-  /* the short status is : action:hostname:current:max */
-  if(msg[0]=='p')
-   action = "portscan";
-  else
-   action  = "attack";
-  
-  msg = msg + sizeof(char)*2; 
-  t = strchr(msg, ':');
-  if(!t)
-   return; 
-  t[0] = 0;
-  hostname = strdup(msg);
-  
-  msg = t + sizeof(char);
-  t = strchr(msg, ':');
-  
-  if(!t)
-   return;
-  t[0]=0;
-  current = strdup(msg);
-  msg = t + sizeof(char);
-  max = atoi(msg);
+ v = harg_get_int(hosts, hostname);
+ if(!v){
+  harg_add_int(hosts, hostname, 1);
+  v = 1;
+  }
+ else
+  {
+  v++;
+  harg_set_int(hosts, hostname, v);
+  }
  }
+ 
  gtkw = arg_get_value(ctrls, "LIST");
  dlist = GTK_LIST(gtkw)->children;
  while(dlist && !flag)
@@ -207,31 +274,12 @@ monitor_list_update(ctrls,msg, short_status)
   {
    GtkWidget * progress_bar;
    gfloat f;
-   GtkWidget * label; 
-   char * lbl;
    
-   if(!strcmp(action, "portscan"))
+   if( strcmp(action, "portscan") == 0 )
     progress_bar = gtk_object_get_data(item, "progress_bar_portscan");
    else
-   {
     progress_bar = gtk_object_get_data(item, "progress_bar_attack");
-    label = gtk_object_get_data(item, "current_plugin");
-   
-   
-    if(current)
-        {
-	 char * str = harg_get_string(plugins_order_table, current);
-	 if(str)
-    	  lbl = label_name(str, progress_bar, arg_get_value(ctrls, "WINDOW"));
-	 else
-	  lbl = label_name("???", progress_bar, arg_get_value(ctrls, "WINDOW"));
-	}
-    else
-        lbl = label_name("???", progress_bar, arg_get_value(ctrls, "WINDOW"));
-	
-    gtk_label_set(GTK_LABEL(label), lbl);
-    free(lbl);
-    }
+    
    gmax = max;
    gcurrent = atoi(current);
    f = (gcurrent/gmax);
@@ -247,17 +295,19 @@ monitor_list_update(ctrls,msg, short_status)
  if(!flag)
  {
  /* the host was not found, we must add one... */
- monitor_add_host(ctrls, hostname, atoi(current));
+ monitor_add_host(ctrls, estrdup(hostname), atoi(current));
  }
-
- if(current)free(current);
+ 
+ efree(&hostname);
+ efree(&action);
+ if(current)efree(&current);
  
 }
  
 /*
  * monitor_remove_host
  */
-void
+static void
 monitor_remove_host(ctrls, host)
  struct arglist * ctrls;
  char * host;
@@ -285,46 +335,16 @@ monitor_remove_host(ctrls, host)
  {
  list = g_list_append(list, item);
  gtk_list_remove_items(GTK_LIST(arg_get_value(ctrls, "LIST")), list);
- gtk_object_set_data(GTK_OBJECT(arg_get_value(ctrls, "LIST")),host,(void*)-1);
+ gtk_object_remove_data(GTK_OBJECT(arg_get_value(ctrls, "LIST")), host);
  g_list_free(list);
  }
 }
 
+ 
+ 
+
 
 
-static char * label_name(char * label, GtkWidget * bar, GtkWidget * window)
-{
- char * ret = emalloc(strlen(label)+1);
- GtkStyle * style;
- int max_len = 195; /* XXX dirty hack : we hardcoded the width of the
- 		       progress bar */
- 
- 
- style = gtk_widget_get_style (window);
- if(!label)label = "???";
- 
- if(gdk_string_width (style->font, label) >  max_len)
- {
-  strncpy(ret, label, strlen(label));
-  while(gdk_string_width (style->font, ret) >  max_len)
-  {
-   if(!strlen(ret))break;
-   ret[strlen(ret)-1]='\0';
-  }
-  if(strlen(ret)>3)
-  {
-  ret[strlen(ret)-1]='.';
-  ret[strlen(ret)-2]='.';
-  ret[strlen(ret)-3]='.';
-  }
- }
- else
- {
-  strncpy(ret, label, strlen(label));
-  }
- return ret;
-}
- 
 /*
  * monitor_add_host
  *
@@ -345,19 +365,18 @@ monitor_add_host(ctrls,hostname,port)
  GtkWidget * button;
  GtkWidget * separator;
  GtkWidget * box, * hbox;
- GtkWidget * other_label;
  GtkStyle  * style;
  GtkWidget * pixmapwid;
  GdkPixmap * pixmap = NULL;
  GdkBitmap * mask;
  GList * dlist;
  GtkWidget * window = arg_get_value(ctrls,"WINDOW");
- char * name;
+
  item = gtk_list_item_new();
  dlist = NULL;
  table = gtk_table_new(4, 3, FALSE);
  gtk_table_set_col_spacings(GTK_TABLE(table), 15);
- gtk_table_set_row_spacings(GTK_TABLE(table),5);
+ gtk_table_set_row_spacings(GTK_TABLE(table),  5);
  
  gtk_container_add(GTK_CONTAINER(item), table);
  
@@ -386,8 +405,8 @@ monitor_add_host(ctrls,hostname,port)
  gtk_box_pack_start(GTK_BOX(box), pixmapwid, FALSE, FALSE, 0);
  gtk_widget_show(pixmapwid);
  }
- name = hostname;
- label = gtk_label_new(name);
+ label = gtk_label_new(hostname);
+ gtk_object_set_data(GTK_OBJECT(item), "label", label);
  gtk_widget_set_usize(label, 150, 15);
  gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);
  gtk_widget_show(label);
@@ -407,10 +426,6 @@ monitor_add_host(ctrls,hostname,port)
  gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);
  gtk_widget_show(label);
  
- label = gtk_label_new("Security check : ");
- gtk_box_pack_end(GTK_BOX(box), label, FALSE, FALSE, 0);
- gtk_widget_show(label);
- 
  /*
   *  Progress bars
   */
@@ -432,12 +447,6 @@ monitor_add_host(ctrls,hostname,port)
  gtk_box_pack_start(GTK_BOX(box), progress_bar_attack, TRUE, TRUE,0);
  gtk_widget_show(progress_bar_attack);
  
- other_label = gtk_label_new("");
- /*
- gtk_table_attach_defaults(GTK_TABLE(table), other_label, 1,2,2,3);
- */
- gtk_box_pack_start(GTK_BOX(box), other_label, TRUE, TRUE, 0);
- gtk_widget_show(other_label);
  
  
  /*
@@ -449,7 +458,7 @@ monitor_add_host(ctrls,hostname,port)
  
  button = gtk_button_new_with_label("Stop");
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
-		     (GtkSignalFunc)monitor_stop_test,hostname);
+		     (GtkSignalFunc)monitor_stop_test, hostname);
  gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
  gtk_widget_show(button);
  
@@ -459,17 +468,16 @@ monitor_add_host(ctrls,hostname,port)
 
 
     
- 
+
  gtk_object_set_data(GTK_OBJECT(item), "hostname", hostname);
  gtk_object_set_data(GTK_OBJECT(item), "progress_bar_attack", progress_bar_attack);
  gtk_object_set_data(GTK_OBJECT(item), "progress_bar_portscan", progress_bar_portscan);
- gtk_object_set_data(GTK_OBJECT(item), "current_plugin", other_label);
  gtk_widget_show(table);
  gtk_widget_show(item);
  if(pixmap)
   gtk_object_set_data(GTK_OBJECT(item), "pixmap", pixmap);
  dlist = g_list_append(dlist, item);
- gtk_object_set_data(GTK_OBJECT(arg_get_value(ctrls, "LIST")),estrdup(hostname),item);
+ gtk_object_set_data(GTK_OBJECT(arg_get_value(ctrls, "LIST")), hostname,item);
  gtk_list_append_items(GTK_LIST(arg_get_value(ctrls, "LIST")), dlist);
  }
  
@@ -494,12 +502,32 @@ monitor_stop_whole_test(w, ctrls)
  GtkWidget * w;
  struct arglist * ctrls;
 {
+ int type = 0, n;
+ char * msg;
+ char buf[32768];
  network_printf( "CLIENT <|> STOP_WHOLE_TEST <|> CLIENT\n");
  efree(&ArgSock);
+ gtk_idle_remove((int)arg_get_value(ctrls, "TAG"));
+ 
+ /*
+  * Read the data remaining...
+  */
+ while(type != MSG_BYE)
+ {
+  bzero(buf, sizeof(buf));
+  network_gets_raw(buf, sizeof(buf) - 1);
+   if( buf[0] == '\0') {
+  	break;
+	}
+  if ((n = strlen (buf)) && buf [n-1] == '\n') buf [n-1] = '\0';
+  msg  = emalloc(strlen(buf)+1);
+  type = parse_server_message(buf, (int)arg_get_value(ctrls, "MONITOR_BACKEND"), msg);
+  efree(&msg);
+ }
+  
  F_nessusd_running = 0;
  gtk_widget_hide(arg_get_value(ctrls, "WINDOW"));
- gtk_idle_remove((int)arg_get_value(ctrls, "TAG"));
- report_tests(arg_get_value(ctrls, "HOSTS"), 0);
+ report_tests_ng((int)arg_get_value(ctrls, "MONITOR_BACKEND"), 0);
  return(FALSE);
 }
 
@@ -517,37 +545,24 @@ monitor_input_callback(ctrls, fd, condition)
     int condition;
 {
   int finished = 0;
-  char buf [4096];
+  char buf [32678];
   char * msg;
   int n, type = -1;
   int interrupted = 0;
-  network_gets_raw(buf, 4095);
-  /* if(strlen(buf))buf[strlen(buf)-1]=0; */
+  network_gets_raw(buf, 32678);
   if ((n = strlen (buf)) && buf [n-1] == '\n') buf [n-1] = '\0';
-#ifdef DEBUGMORE
-  fprintf(stderr, "%s\n", buf);
-#endif
-#ifdef ENABLE_CRYPTO_LAYER
-  if (buf [0] == '\0') {
-    n = (errno == 0) ? io_ctrl (GlobalSocket, IO_EOF_STATE, 0, 0) : -1 ;
-    if (n == 0) return ;
-    if (n  > 0) {
-    	/*show_error("Error : nessusd shut down the connection during the test !");*/
-    	interrupted++;
-	goto scan_finished;
-	}
-    sprintf (buf, "COMMUNICATION ERROR %u: %s", errno, peks_strerr (errno));
-    show_error (buf) ;
-    goto scan_finished;
-  }
-#endif
-  msg = emalloc(strlen(buf)+1);
-  if(strlen(buf))type=parse_server_message(buf, arg_get_value(ctrls, "HOSTS"), msg);
-   else {
-   	/*show_error("Error : nessusd shut down the connection during the test !");*/
-	interrupted++;
-   	goto scan_finished;
+
+  if( buf[0] == '\0') {
+  	if(!is_server_present(GlobalSocket))
+	{
+  	 interrupted++;
+   	 goto scan_finished;
 	}
+        else return;
+       }	
+  msg  = emalloc(strlen(buf)+1);
+  type = parse_server_message(buf, (int)arg_get_value(ctrls, "MONITOR_BACKEND"), msg);
+ 
   switch(type)
   {
   	case MSG_BYE : 
@@ -567,7 +582,7 @@ monitor_input_callback(ctrls, fd, condition)
 		monitor_remove_host(ctrls, msg);
 		break;	
   }
-  bzero(buf, 1024);
+  bzero(buf, sizeof(buf));
   efree(&msg);
   
   if(finished)
@@ -577,7 +592,7 @@ monitor_input_callback(ctrls, fd, condition)
       gtk_widget_hide(arg_get_value(ctrls, "WINDOW"));
       gtk_idle_remove((int)arg_get_value(ctrls, "TAG"));
       gtk_widget_destroy(arg_get_value(ctrls, "WINDOW"));
-      report_tests(arg_get_value(ctrls, "HOSTS"), interrupted);
+      report_tests_ng((int)arg_get_value(ctrls, "MONITOR_BACKEND"), interrupted);
     }
 }
  
@@ -603,7 +618,9 @@ static void build_plugins_order_table(order)
  int * plugins_order_table_int;
  t = order;
 
- efree(&plugins_order_table);
+ if(plugins_order_table)
+  harg_close(plugins_order_table);
+ 
  
  while((t = strchr(t+1, ';')))num++;
  plugins_order_table_int = emalloc((num+1)*sizeof(int));
