@@ -28,22 +28,15 @@
 #endif
 
 #define DEFAULT_SERVER "localhost"
-#define PROTO_NAME "< NTP/1.2 >\n"
+#define PROTO_NAME "< NTP/1.2 >< plugins_cve_id plugins_version plugins_bugtraq_id timestamps dependencies >\n"
 
-#ifdef ENABLE_CRYPTO_LAYER
-#define DEFAULT_CIPHER    "twofish/ripemd160:3" /* alternatively: "none" */
-#define CIPHER_PROTO_NAME "< NSP/0.3 >\n"
-#define NESSUS_AUTH_METH  3 /* auth scheme, either 1 or 3 */
-#ifdef _WIN32
-#define NESSUS_KEYFILE    "~/nessus-keys.cfg"
-#define NESSUS_RCFILE     "~/nessusrc.cfg"
+#ifdef _CYGWIN_
+extern char * NESSUS_RCFILE;
+extern int init_directories;
 #else
-#define NESSUS_KEYFILE    "~/.nessus.keys"
 #define NESSUS_RCFILE     "~/.nessusrc"
 #endif
-#define NESSUS_SIGKEYLEN  1024
-#define NESSUS_MINKEYLEN   512 /* used to check options, only */
-#endif
+
 
 char * connect_to_nessusd(char *, int, char *, char *);
 
