@@ -1,10 +1,10 @@
 /* Nessus
- * Copyright (C) 1998, 1999 Renaud Deraison
+ * Copyright (C) 1998 - 2001 Renaud Deraison
+ *
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2,
+ * as published by the Free Software Foundation
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,17 +15,200 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
+ * In addition, as a special exception, Renaud Deraison
+ * gives permission to link the code of this program with any
+ * version of the OpenSSL library which is distributed under a
+ * license identical to that listed in the included COPYING.OpenSSL
+ * file, and distribute linked combinations including the two.
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * this file, you may extend this exception to your version of the
+ * file, but you are not obligated to do so.  If you do not wish to
+ * do so, delete this exception statement from your version.
  */
  
 #include <includes.h>
 #include "report.h"
 #include "error_dialog.h"
+#include "backend.h"
+#include "data_mining.h"
+#include "report_utils.h"
+
+/*
+ * nessusd does not convert the subnet by itself, so we create
+ * this record on this fly
+ */
+static char *
+__host2subnet(host)
+ char * host;
+{
+ static char subnet[1024];
+ struct in_addr ia;
+ bzero(subnet, sizeof(subnet));
+ if(inet_aton(host, &ia) == 0)
+ {
+  char * t;
+  /*
+   * Not an IP
+   */
+  t = strchr(host, '.');
+  if(t)
+	++t;
+  else
+	t = host;
+   strncpy(subnet, t, sizeof(subnet) - 1);
+   return subnet;
+ }
+ else
+ {
+  /*
+   * Is an IP
+   */
+  char * t = strrchr(host, '.');
+  if(t)t[0] = '\0';
+  strncpy(subnet, host, sizeof(subnet) - 1);
+  if(t)t[0] = '.';
+  return subnet;
+ }
+}
+
+
+
+int backend_to_nsr(int, char *);
+int nsr_to_backend(char*);
+
 
 int arglist_to_file(struct arglist * , char * );
 int file_to_arglist(struct arglist ** , char * );
 int arglist_to_html(struct arglist *, char *);
 
 
+
+int nsr_to_backend(filename)
+ char * filename;
+{
+ FILE * f = fopen(filename, "r");
+ char buf[32768];
+ int be;
+ if(!f)
+ {
+  perror("fopen ");
+  show_error("Could not open report");
+  return -1;
+ }
+
+ be = backend_init(NULL);
+ bzero(buf, sizeof(buf));
+ while(fgets(buf, sizeof(buf) - 1, f))
+ {
+  char * t = strchr(buf, '|');
+  if(t)
+  {
+   char* subnet;
+   char* hostname = buf;
+   char* port;
+   char* script_id = "";
+   char* severity = "";
+   char* data = "";
+   
+   t[0] = '\0';
+   subnet = __host2subnet(hostname);
+   port = &(t[1]);
+   t = strchr(port, '|');
+   if(!t)
+   {
+    /*
+     * It was a port
+     */
+     backend_insert_report_port(be, subnet, hostname, port);
+   }
+   else
+   {
+    t[0] = '\0';
+    script_id = &(t[1]);
+    t = strchr(script_id, '|');
+    if(t)
+    {
+     t[0] = '\0';
+     severity = &(t[1]);
+     t = strchr(severity, '|');
+     if(t)
+     {
+      t[0] = '\0';
+      data = &(t[1]);
+     }
+    }
+    
+    while((t = strchr(data, ';')))t[0] = '\n';
+    if(!strcmp(severity, "INFO"))severity = "Security Note";
+    else if(!strcmp(severity, "NOTE"))severity = "Security Warning";
+    else if(!strcmp(severity, "REPORT"))severity ="Security Hole";
+    backend_insert_report_data(be, subnet, hostname, port, script_id, severity, data);
+  
+   }
+  }
+ }
+ fclose(f);
+ return be;
+}
+
+int backend_to_nsr(be, filename)
+ int be;
+ char * filename;
+{
+ int fd = open(filename, O_RDWR|O_CREAT|O_TRUNC, 0600);
+ struct subset  * sq, *q;
+ cmp_func_t cmp[] = {safe_strcmp};
+ if(fd < 0)
+ {
+  perror(filename);
+  show_error(strerror(errno));
+  return -1;
+ }
+
+ 
+ /*
+  * Signature
+  */
+
+ 
+ sq = q = subset_uniq(subset_sort(query_backend(be, "SELECT subnet FROM results"), 0, 0, cmp), 0);
+ while(q)
+ {
+  struct subset * sr, *r;
+  sr = r = query_backend(be, "SELECT host,port,plugin_id,severity,report FROM results WHERE subnet = '%s'", subset_value(q));
+  while(r)
+  {
+   int i;
+   for(i=0;i<5;i++)
+   {
+    char * data = subset_nth_value(r, i);
+    char * t;
+  
+    while((t = strchr(data, '\n')))t[0] = ';';
+    if(i == 3) 
+    { 
+     if(strstr(data, "Hole"))data = "REPORT";
+     else if(strstr(data, "Warning"))data = "INFO";
+     else if(strstr(data, "Note"))data = "NOTE";
+    }
+    write(fd, data, strlen(data));
+ 
+    if(subset_nth_value(r, i+1))write(fd, "|", 1);
+    else break;
+   }
+  write(fd, "\n", 1);
+  r = subset_next(r);
+  }
+  subset_free(sr);
+  q = subset_next(q);
+ }
+ subset_free(sq);
+ close(fd);
+ return 0; 
+}
+
+
 int 
 arglist_to_file(hosts, filename)
  struct arglist * hosts;
@@ -57,7 +240,9 @@ arglist_to_file(hosts, filename)
     
     port = ports->name;
     report = arg_get_value(ports->value, "REPORT");
-    if(report)while(report && report->next)
+    if(report)while(report->next)
+     {
+     if(report->name != NULL)
      {
      char * t;
      desc = emalloc(strlen(report->value)+1);
@@ -69,11 +254,14 @@ arglist_to_file(hosts, filename)
 					  report->name:"", 
 					  desc);
      efree(&desc);
+     }
      report = report->next;
      } 
    info = arg_get_value(ports->value, "INFO");
-   if(info)while(info && info->next)
+   if(info)while(info->next)
     {
+     if(info->name != NULL)
+     {
      char * t;
      desc = emalloc(strlen(info->value)+1);
      strncpy(desc, info->value, strlen(info->value));
@@ -84,12 +272,15 @@ arglist_to_file(hosts, filename)
 					info->name:"", 
 					desc);
      efree(&desc);
+     }
      info = info->next;
     }
 
    note = arg_get_value(ports->value, "NOTE");
-   if(note)while(note && note->next)
+   if(note)while(note->next)
     {
+     if(note->name != NULL)
+     {
      char * t;
      desc = emalloc(strlen(note->value)+1);
      strncpy(desc, note->value, strlen(note->value));
@@ -100,6 +291,7 @@ arglist_to_file(hosts, filename)
 					note->name:"", 
 					desc);
      efree(&desc);
+     }
      note = note->next;
     }
 
@@ -128,7 +320,7 @@ file_to_arglist(hosts, filename)
  char * filename;
 {
  FILE * fd;
- char buf[4096];
+ char buf[65535];
  harglst * hhosts = harg_create(65000);
  hargwalk * hw;
  struct arglist * current_host = NULL;
@@ -136,12 +328,16 @@ file_to_arglist(hosts, filename)
  int line = 0;
  char * key;
  struct arglist *nhosts;
+ 
+ if(!filename)
+  return -1;
+  
  if(!strcmp(filename, "-"))fd = stdin;
  else fd = fopen(filename, "r");
  if(!fd)return(-1);
  
- bzero(buf, 4095);
- while(fgets(buf, 4095, fd) && !feof(fd))
+ bzero(buf, sizeof (buf));
+ while(fgets(buf, sizeof (buf)-1, fd) && !feof(fd))
  {
   char * buffer = NULL;
   struct arglist * host = NULL;
@@ -157,6 +353,7 @@ file_to_arglist(hosts, filename)
   t = strchr(buf, '|');
   if(!t){
   	fprintf(stderr, "** Parse error line %d - ignoring\n", line);
+	fprintf(stderr, "buf = <%s>\n", buf);
   	continue;
 	}
 
@@ -215,7 +412,7 @@ file_to_arglist(hosts, filename)
   efree(&buffer);
   
   if(!t2[1]){
-  	bzero(buf, 4095);
+  	bzero(buf, sizeof (buf));
 	continue; /* port is open, that's all. */
  	}
    t = t2+sizeof(char);
@@ -226,7 +423,7 @@ file_to_arglist(hosts, filename)
   t = strchr(t2, '|');
   if(!t)continue;
   t[0]=0;
-  if(atoi(t2) > 10000){
+  if(atoi(t2) > 1000){
 	id = strdup(t2);
 	}
   
@@ -235,9 +432,7 @@ file_to_arglist(hosts, filename)
   if(!t2)continue;
   t2[0]=0;
    
-  if(!strcmp(t, "INFO")) id = "INFO";
-  else if(!strcmp(t, "NOTE")) id = "NOTE";
-  else if(!strcmp(t, "REPORT")) id = "REPORT";
+ 
   
   buffer = strdup(t);
   content = arg_get_value(port, buffer);
@@ -256,10 +451,12 @@ file_to_arglist(hosts, filename)
   
   buffer = emalloc(strlen(t2)+1);
   strncpy(buffer, t2, strlen(t2));
+
+
   arg_add_value(content, id, ARG_STRING, strlen(t2),buffer);
-  bzero(buf, 4095);
+  efree(&id);
+  bzero(buf, sizeof (buf));
  }
- efree(&buf);
  fclose(fd);
  
  /*
@@ -271,10 +468,8 @@ file_to_arglist(hosts, filename)
  {
   struct arglist * new = emalloc(sizeof(struct arglist));
   struct arglist * h = harg_get_ptr(hhosts, key);
-  new->name = strdup(key);
-  new->type = ARG_ARGLIST;
-  new->length = -1;
-  new->value = h;
+  arg_add_value(new, key, ARG_ARGLIST, -1, h);
+  efree(&new->next);
   new->next = nhosts;
   nhosts = new;
  }
