@@ -2,9 +2,8 @@
  * Copyright (C) 1998,1999,2000 Renaud Deraison
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2,
+ * as published by the Free Software Foundation
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,6 +14,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
+ * In addition, as a special exception, Renaud Deraison
+ * gives permission to link the code of this program with any
+ * version of the OpenSSL library which is distributed under a
+ * license identical to that listed in the included COPYING.OpenSSL
+ * file, and distribute linked combinations including the two.
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * this file, you may extend this exception to your version of the
+ * file, but you are not obligated to do so.  If you do not wish to
+ * do so, delete this exception statement from your version.
  *
  */
  
@@ -22,11 +31,96 @@
 #include <includes.h>
 
 #ifdef USE_GTK 
+#include "gtk-compat.h"
 #include <gtk/gtk.h>
 
 #include "xstuff.h"
 #include "globals.h"
 
+
+static void
+show_deps(foo, name)
+ GtkWidget * foo;
+ char * name;
+{
+ GtkWidget * window;
+ GtkWidget * w;
+ GtkWidget * box;
+ struct arglist * deps;
+ char * lbl = emalloc(strlen(name) + 255);
+ int label_size = 1024;
+ char * label;
+ sprintf(lbl, "Dependencies of '%s'", name);
+
+ window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+ gtk_window_set_title(GTK_WINDOW(window), lbl);
+
+ 
+ gtk_container_border_width(GTK_CONTAINER(window), 10);
+ gtk_signal_connect(GTK_OBJECT(window), "destroy", 
+                GTK_SIGNAL_FUNC(close_window), window);
+ 
+ box = gtk_vbox_new(FALSE, 5);
+ gtk_container_add(GTK_CONTAINER(window), box);
+ gtk_widget_show(box);
+ 
+ w = gtk_label_new(lbl);
+ efree(&lbl);
+ gtk_box_pack_start(GTK_BOX(box), w, TRUE, TRUE, 5);
+ gtk_widget_show(w);
+ 
+ w = gtk_hseparator_new();
+ gtk_box_pack_start(GTK_BOX(box), w, TRUE, TRUE, 5);
+ gtk_widget_show(w);
+
+ 
+ deps = arg_get_value(Dependencies, name);
+ if(!deps)
+  {
+   return; /* XXX */
+  } 
+ label = emalloc(label_size);
+ 
+ while(deps->next)
+ {
+ struct arglist * p = arg_get_value(Plugins, deps->name);
+ 
+ if(p == NULL) 
+ 	p = arg_get_value(Scanners, deps->name);
+ if(p)
+ {
+  char * family = arg_get_value(p, "FAMILY");
+  if(strlen(deps->name) + strlen(family) + 255 > label_size)
+  { 
+   label_size *= 2;
+   label = erealloc(label, label_size);
+  }
+  strncpy(label, deps->name, label_size);
+  strncat(label, " (", label_size);
+  strncat(label, family, label_size);
+  strncat(label, "), currently ", label_size);
+  if(arg_get_value(p, "ENABLED"))
+  	strncat(label, "enabled", label_size);
+  else
+  	strncat(label, "disabled", label_size);
+ }
+ else strncpy(label, deps->name, label_size);
+ w = gtk_label_new(label);
+ gtk_box_pack_start(GTK_BOX(box), w,   TRUE, TRUE, 2);
+ gtk_widget_show(w);
+ deps = deps->next;
+ }
+ 
+ efree(&label);
+ w = gtk_button_new_with_label("Close");
+ gtk_box_pack_start(GTK_BOX(box), w, TRUE, TRUE, 3);
+ gtk_signal_connect(GTK_OBJECT(w), "clicked",GTK_SIGNAL_FUNC(close_window), window);
+ gtk_widget_show(w);
+
+
+ gtk_widget_show(window);
+}
+
 static void 
 do_set_timeout(b, ctrls)
  GtkWidget* b;
@@ -72,6 +166,7 @@ set_timeout_build_window(id, cur_to, def_to)
  GtkWidget * box, * hbox;
  struct arglist * ctrls = emalloc(sizeof(*ctrls));
  
+
  window = gtk_window_new(GTK_WINDOW_DIALOG);
  gtk_window_set_title(GTK_WINDOW(window), "Set plugin timeout...");
  gtk_container_border_width(GTK_CONTAINER(window), 10);
@@ -197,7 +292,11 @@ plugin_info_window_setup(res, pluginname)
  }
  buf = emalloc(4095);
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+#if GTK_VERSION > 10
  gtk_window_set_default_size(GTK_WINDOW(window), 400,500);
+#else
+ gtk_widget_set_usize(GTK_WIDGET(window), 400, 500);
+#endif
  gtk_container_border_width(GTK_CONTAINER(window),10);
  gtk_window_set_title(GTK_WINDOW(window), pluginname);
  
@@ -216,7 +315,12 @@ plugin_info_window_setup(res, pluginname)
  gtk_widget_show(separator);
  
  
- sprintf(buf, "Family : %s", plug_get_family(plugin));
+ sprintf(buf, "Family : %s", (char*)arg_get_value(plugin, "FAMILY"));
+ label = gtk_label_new(buf);
+ gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE,0);
+ gtk_widget_show(label);
+ 
+ sprintf(buf, "Category : %s", (char*)arg_get_value(plugin, "CATEGORY"));
  label = gtk_label_new(buf);
  gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE,0);
  gtk_widget_show(label);
@@ -226,6 +330,12 @@ plugin_info_window_setup(res, pluginname)
  gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE,0);
  gtk_widget_show(label);
  
+ label = gtk_label_new(arg_get_value(plugin, "VERSION"));
+ gtk_box_pack_start(GTK_BOX(box), label, FALSE,FALSE,0);
+ gtk_widget_show(label);
+ 
+ 
+ 
  
  separator = gtk_hseparator_new();
  gtk_box_pack_start(GTK_BOX(box), separator, FALSE, FALSE,0);
@@ -268,6 +378,15 @@ plugin_info_window_setup(res, pluginname)
  gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 5);
  gtk_widget_show(button);
  
+ button = gtk_button_new_with_label("Show dependencies");
+ gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(show_deps), arg_get_value(plugin, "NAME"));
+ gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 5);
+ gtk_widget_show(button);
+ if(!arg_get_value(Dependencies, pluginname))
+ {
+  gtk_widget_set_sensitive(button,FALSE);  
+ }
+ 
  button = gtk_button_new_with_label("Close");
  gtk_signal_connect(GTK_OBJECT(button), "clicked",GTK_SIGNAL_FUNC(close_window), window);
  gtk_box_pack_end(GTK_BOX(box), button, FALSE, FALSE,5);
