@@ -1,10 +1,9 @@
 /* Nessus
- * Copyright (C) 1998 Renaud Deraison
+ * Copyright (C) 1998 - 2001 Renaud Deraison
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2,
+ * as published by the Free Software Foundation
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,6 +14,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
+ * In addition, as a special exception, Renaud Deraison
+ * gives permission to link the code of this program with any
+ * version of the OpenSSL library which is distributed under a
+ * license identical to that listed in the included COPYING.OpenSSL
+ * file, and distribute linked combinations including the two.
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * this file, you may extend this exception to your version of the
+ * file, but you are not obligated to do so.  If you do not wish to
+ * do so, delete this exception statement from your version.
+ *
  * Preferences  -- maps the content of the nessusd.conf file to memory
  *
  */
@@ -27,9 +37,31 @@
 static int preferences_new();
 static int prefs_buffer_parse(char *, struct arglist *);
 static int prefs_add_subcategory(struct arglist *, FILE *);
-static char * preferences_get_filename();
 static void new_pluginset(struct arglist *, struct arglist *);
 
+static int
+plugin_id(plugin)
+ struct arglist * plugin;
+{
+ return (int)arg_get_value(plugin, "ID");
+}
+
+char *
+plugin_asc_id(plugin)
+ struct arglist * plugin;
+{
+ static char asc_id[21];
+ char * ret;
+ if((ret = arg_get_value(plugin, "ASC_ID")))
+  return ret;
+ else
+ {
+  bzero(asc_id, sizeof(asc_id));
+  sprintf(asc_id, "%d", plugin_id(plugin));
+  arg_add_value(plugin, "ASC_ID", ARG_STRING, strlen(asc_id), asc_id);
+  return asc_id;
+ }
+}
 
 
 
@@ -41,9 +73,8 @@ int preferences_init(prefs)
   *prefs = emalloc(sizeof(struct arglist));
   filename = preferences_get_filename();
   result = preferences_process(filename, *prefs);
-#ifndef ENABLE_CRYPTO_LAYER
+  /*ENABLE_CRYPTO_LAYER*/
   efree(&filename);
-#endif
   if (result && getenv ("NESSUSHOME") == 0)
     show_error_and_wait (CANNOT_SET_HOMEVAR);
   return(result);
@@ -56,33 +87,14 @@ int preferences_init(prefs)
  */
 
 
-static char * 
+char * 
 preferences_get_filename()
 {
- if(Alt_rcfile)return estrdup(Alt_rcfile);
+ if(Alt_rcfile)
+ 	return estrdup(Alt_rcfile);
  else
   {
-#ifdef ENABLE_CRYPTO_LAYER
-  char *t, *s = peks_get_dirfile (NESSUS_RCFILE);
-  if (s == 0) 
-    return 0 ;
-  /* peks uses another memory management */
-  t = estrdup (s);
-  xfree (s);
-#ifdef _WIN32
-  /* on Windows we have no HOME varable set, by default */
-  if (getenv("HOME") == 0 && (s = peks_get_homedir (0)) != 0) {
-    char *e = strcat (strcpy (pmalloc (strlen (s) + 10), "HOME="), s);
-#   ifdef HAVE__PUTENV
-    _putenv (e);
-#   else
-    putenv (e);
-#   endif
-    xfree (e);
-  }
-#endif /* _WIN32 */
-  return t;
-#else
+    /*ENABLE_CRYPTO_LAYER*/
   char* home;
   char * ret;
   struct passwd * pwd;
@@ -102,56 +114,37 @@ preferences_get_filename()
    return(ret);
   } 
   return(NULL);
-#endif
  }
 }
 
 
-
-static char * 
-preferences_get_dirname()
+char * 
+preferences_get_altname(ext)
+     const char	*ext;
 {
-#ifdef ENABLE_CRYPTO_LAYER
-  char *t, *s = peks_get_dirfile (NESSUS_RCFILE);
-  if (s == 0) 
-    return 0 ;
-  /* peks uses another memory management */
-  t = estrdup (s);
-  xfree (s);
-#ifdef _WIN32
-  /* on Windows we have no HOME varable set, by default */
-  if (getenv("HOME") == 0 && (s = peks_get_homedir (0)) != 0) {
-    char *e = strcat (strcpy (pmalloc (strlen (s) + 10), "HOME="), s);
-#   ifdef HAVE__PUTENV
-    _putenv (e);
-#   else
-    putenv (e);
-#   endif
-    xfree (e);
-  }
-#endif /* _WIN32 */
-  return t;
-#else
   char* home;
   char * ret;
   struct passwd * pwd;
+  int	l = (ext == NULL) ? 0 : strlen(ext) + 1;
   
   home = getenv("HOME");
-  if(home)
-  {
-   ret = emalloc(strlen(home)+strlen("/.nessusrc")+1);
-   sprintf(ret, "%s/.nessusrc", home);
-   return(ret);
-  }
-  pwd = getpwuid(getuid());
-  if (pwd && pwd->pw_dir)
-  {
-   ret = emalloc(strlen(pwd->pw_dir)+strlen("/.nessusrc")+1);
-   sprintf(ret, "%s/.nessusrc", pwd->pw_dir);
-   return(ret);
-  } 
-  return(NULL);
-#endif
+  if(home == NULL)
+    {
+      pwd = getpwuid(getuid());
+      if (pwd != NULL && pwd->pw_dir != NULL)
+	home = pwd->pw_dir;
+      else
+	return NULL;
+    }
+
+  ret = emalloc(strlen(home)+strlen("/.nessusrc")+l+1);
+  if (ret == NULL)
+    return NULL;
+  if (ext == NULL)
+    sprintf(ret, "%s/.nessusrc", home);
+  else
+    sprintf(ret, "%s/.nessusrc.%s", home, ext);
+  return ret;
 }
 
 
@@ -168,13 +161,17 @@ static int preferences_new()
 	return -1;
       }
   fprintf(f,"# Nessus Client Preferences File\n\n");
+  fprintf(f, "trusted_ca = %s/cacert.pem\n", NESSUSD_CA);
   fprintf(f, "begin(SCANNER_SET)\n");
-  fprintf(f, "Ping the remote host = yes\n");
-  fprintf(f, "TCP Ping the remote host = no\n");
-  fprintf(f, "TCP SYN scan = no\n");
-  fprintf(f, "FTP bounce scan = no\n");
-  fprintf(f, "Nmap tcp connect() scan = no\n");
-  fprintf(f, "Nmap = yes\n");
+  fprintf(f, "10180 = yes\n");
+  fprintf(f, "10277 = no\n");
+  fprintf(f, "10278 = no\n");
+  fprintf(f, "10331 = no\n");
+  fprintf(f, "10335 = yes\n");
+  fprintf(f, "10841 = no\n");
+  fprintf(f, "10336 = no\n");
+  fprintf(f, "10796 = no\n");
+  fprintf(f, "11219 = no\n");
   fprintf(f, "end(SCANNER_SET)\n\n");
 
   fclose(f);
@@ -262,10 +259,10 @@ prefs_add_subcategory(arglist, fd)
  struct arglist * arglist;
  FILE * fd;
 {
- char * buffer = emalloc(255);
+ char * buffer = emalloc(4096);
  int flag = 0;
  
- while(!flag && !feof(fd) && fgets(buffer, 254,fd))
+ while(!flag && !feof(fd) && fgets(buffer, 4096,fd))
  {
   if(!strlen(buffer))return(1);
   if((!strcmp(buffer, "end\n"))||(!strncmp(buffer, "end(", 4)))flag = 1;
@@ -302,6 +299,13 @@ prefs_buffer_parse(buffer, arglist)
   /* char to int conversion if necessary */
   if(!strcmp(t, "yes"))val = 1;
   if(!strcmp(t, "no"))val = 0;
+  
+  if(!strcmp(opt, "paranoia_level"))
+  {
+   arg_add_value(arglist, opt, ARG_INT, sizeof(int), (void*)atoi(t));
+  }
+  else
+  {
   if(val == -1)
   {
    /* the string is not 'yes' nor 'no' so we take it as a string */
@@ -310,6 +314,7 @@ prefs_buffer_parse(buffer, arglist)
    arg_add_value(arglist, opt, ARG_STRING, strlen(value), value);
   }
   else arg_add_value(arglist, opt, ARG_INT, sizeof(int), (void *)val);
+  }
   return(0);
  }
  else return(1);
@@ -333,12 +338,18 @@ preferences_save_fname(filename, plugins)
  chmod(filename, 0600);
  fprintf(fd, "# This file was automagically created by nessus\n");
  t = Prefs;
+
  while(t && t->next)
  {
   if((int)t->type==ARG_INT)
-   fprintf(fd, "%s = %s\n", t->name, t->value?"yes":"no");
+   {
+   if(!strcmp(t->name, "paranoia_level"))
+    fprintf(fd, "%s = %d\n", t->name, (int)t->value);
+   else
+    fprintf(fd, "%s = %s\n", t->name, t->value?"yes":"no");
+   }
   else if((t->type == ARG_STRING)&&(strlen(t->value)))
-  	fprintf(fd, "%s = %s\n", t->name, (char *)t->value);
+  	fprintf(fd, "%s = %s\n",t->name, (char *)t->value);
   t = t->next;
  }
  
@@ -359,8 +370,19 @@ preferences_save_fname(filename, plugins)
       }
     if(v->type==ARG_INT)
      fprintf(fd, " %s = %s\n", v->name, v->value?"yes":"no");
-    else if((v->type == ARG_STRING)&&v->value && strlen(v->value))
-    	fprintf(fd, " %s = %s\n", v->name, (char *)v->value);
+    else 
+     {
+      if((v->type == ARG_STRING)&&v->value)
+       {
+        if(!strcmp(t->name, "PLUGINS_PREFS"))
+    		fprintf(fd, " %s = %s\n", v->name, (char *)v->value);
+        else 
+	{
+	 if(strlen((char*)v->value))
+		fprintf(fd, " %s = %s\n", v->name, (char *)v->value);
+	}
+       }
+     }
      v = v->next;
    }
   fprintf(fd, "end(%s)\n\n", t->name);
@@ -393,7 +415,7 @@ pluginset_apply(plugins, name)
   
   while(!flag && t && t->next)
   {
-   if(!strcmp(plugins->name, t->name))
+   if(!strcmp(plugin_asc_id(plugins->value), t->name))
     {
      plug_set_launch(plugins->value, (int)t->value);
      flag = 1;
@@ -404,7 +426,10 @@ pluginset_apply(plugins, name)
    if(!flag)
    {
     char * c = arg_get_value(plugins->value, "CATEGORY");
-    if(c && !strcmp(c, "denial"))plug_set_launch(plugins->value, 0);
+    printf("%s is unknown to us\n", plugins->name);
+    if(c && (!strcmp(c, "denial")||
+	     ! strcmp(c, "kill_host") ||
+    	     !strcmp(c, "destructive_attack")))plug_set_launch(plugins->value, 0);
     else plug_set_launch(plugins->value, 1);
     ret++;
    }
@@ -434,7 +459,7 @@ pluginset_reload(plugins, scanners)
   t = arg_get_value(Prefs, "PLUGIN_SET");
   if(t)while(!flag && t && t->next)
   {
-   if(!strcmp(plugins->name, t->name))
+   if(!strcmp(plugin_asc_id(plugins->value), t->name))
    {
     t->value = (void *)plug_get_launch(plugins->value);
     flag = 1;
@@ -457,7 +482,7 @@ pluginset_reload(plugins, scanners)
   t = arg_get_value(Prefs, "SCANNER_SET");
   if(t)while(!flag && t && t->next)
   {
-   if(!strcmp(scanners->name, t->name))
+   if(!strcmp(plugin_asc_id(scanners->value),t->name))
    {
     t->value = (void *)plug_get_launch(scanners->value);
     flag = 1;
@@ -475,7 +500,9 @@ static void new_pluginset(pluginset, plugins)
 {
  while(plugins && plugins->next)
  {
-  arg_add_value(pluginset, plugins->name, ARG_INT, sizeof(int),
+  char * name = emalloc(20);
+  sprintf(name, "%d", plugin_id(plugins->value));
+  arg_add_value(pluginset, name, ARG_INT, sizeof(int),
   	 (void *)plug_get_launch(plugins->value));
   plugins = plugins->next;
  }
