@@ -15,6 +15,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
+ * In addition, as a special exception, Renaud Deraison
+ * gives permission to link the code of this program with any
+ * version of the OpenSSL library which is distributed under a
+ * license identical to that listed in the included COPYING.OpenSSL
+ * file, and distribute linked combinations including the two.
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * this file, you may extend this exception to your version of the
+ * file, but you are not obligated to do so.  If you do not wish to
+ * do so, delete this exception statement from your version.
  */
 
 #include <includes.h>
@@ -23,6 +33,7 @@
 #include <gtk/gtk.h>
 #include "../xstuff.h"
 #include "../preferences.h"
+#include "../sighand.h"
 #include "prefs_dialog_scan_opt.h"
 #include "prefs_target.h"
 #include "prefs_dialog_user.h"
@@ -33,6 +44,7 @@
 #include "prefs_dialog.h"
 #include "prefs_about.h"
 
+
 #ifdef ENABLE_SAVE_KB
 #include "prefs_kb.h"
 #endif
@@ -83,7 +95,11 @@ prefs_dialog_setup(widget, preferences)
 /*
  * We draw the window ....
  */
+#if GTK_VERSION >= 20
+ window = gtk_dialog_new();
+#else 
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+#endif 
  gtk_signal_connect(GTK_OBJECT(window), "delete_event",
                      (GtkSignalFunc)close_display,NULL);
                      
@@ -94,17 +110,19 @@ prefs_dialog_setup(widget, preferences)
  /*
   * We put a vbox in it...
   */
-  
+#if GTK_VERSION < 20  
  box = gtk_vbox_new(FALSE, 10);
  gtk_container_add(GTK_CONTAINER(window), box);
  gtk_widget_show(box);
- 
+#else
+ box = GTK_DIALOG(window)->vbox; 
+#endif 
 /* 
  * We set up the notebook
  */
  notebook = gtk_notebook_new();
  gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);
- gtk_box_pack_start(GTK_BOX(box), notebook, TRUE, TRUE, 0);
+ gtk_box_pack_start(GTK_BOX(box), notebook, TRUE, TRUE, 5);
  gtk_widget_show(notebook);
  arg_add_value(ctrls, "NOTEBOOK", ARG_PTR, -1, notebook);
 
@@ -115,7 +133,7 @@ prefs_dialog_setup(widget, preferences)
  prefs_target = prefs_dialog_target(preferences);
  prefs_user = prefs_dialog_user(preferences);
  prefs_auth = prefs_dialog_auth(window);
- prefs_plugins = prefs_dialog_plugins();
+ prefs_plugins = prefs_dialog_plugins(window);
  prefs_plugins_prefs = prefs_dialog_plugins_prefs();
 #ifdef ENABLE_SAVE_KB
  prefs_kb = prefs_dialog_kb(); 
@@ -213,7 +231,7 @@ prefs_dialog_setup(widget, preferences)
 
  cancel = gtk_button_new_with_label("Quit");
  gtk_signal_connect(GTK_OBJECT(cancel), "clicked",
- 	GTK_SIGNAL_FUNC(exit), window);
+ 	GTK_SIGNAL_FUNC(nessus_exit), NULL);
  gtk_box_pack_end(GTK_BOX(hbox), cancel,TRUE, TRUE, 0);
  gtk_widget_show(cancel);
  
@@ -238,12 +256,32 @@ void prefs_dialog_set_defaults(ctrls, preferences)
  int flag = 0;
  GtkWidget * gtkw;
  
+ 
+ serv = arg_get_value(preferences, "SERVER_PREFS");
+ if(!serv)return;
+ 
+ 
+ t = arg_get_value(ctrls, "PLUGINS");
+ gtkw = arg_get_value(t, "ENABLE_DEPS_AT_RUNTIME");
+ if(gtkw)
+ {
+  v = arg_get_value(serv, "auto_enable_dependencies");
+  if(arg_get_type(serv, "auto_enable_dependencies") == ARG_INT)
+  {
+      char * s = strdup(v ? "yes":"no");
+      arg_set_type(serv, "auto_enable_dependencies", ARG_STRING);
+      arg_set_value(serv, "auto_enable_dependencies", strlen(s), s);
+      v = s;
+  } 
+  if(v && !strcmp(v, "yes"))
+  	GTK_TOGGLE_BUTTON(gtkw)->active = TRUE;
+  else
+  	GTK_TOGGLE_BUTTON(gtkw)->active = FALSE;
+ }
  /*
   *  Host expansion options
   */
  t = arg_get_value(ctrls, "TARGET");
- serv = arg_get_value(preferences, "SERVER_PREFS");
- if(!serv)return;
  v = arg_get_value(serv, "host_expansion");
  if(!v)
   flag |= EX_NONE;
@@ -335,11 +373,45 @@ void prefs_dialog_set_defaults(ctrls, preferences)
    arg_set_value(serv, "optimize_test", strlen(s), s);
    arg_set_type(serv, "optimize_test", ARG_STRING);
   }
-  
   v = arg_get_value(serv, "optimize_test");
   if(v)gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(gtkw), !strcmp(v, "yes"));
   else gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(gtkw), 1);
  
+ 
+  gtkw = arg_get_value(t, "SAFE_CHECKS");
+  v = arg_get_value(serv, "safe_checks");
+  if(arg_get_type(serv, "safe_checks")==ARG_INT)
+  {
+   char * s = emalloc(4);
+   if(v)strncpy(s, "yes", 3);
+   else strncpy(s, "no", 2);
+   
+   arg_set_value(serv, "safe_checks", strlen(s), s);
+   arg_set_type(serv, "safe_checks", ARG_STRING);
+  }
+  
+   v = arg_get_value(serv, "safe_checks");
+  if(v)gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(gtkw), !strcmp(v, "yes"));
+  else gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(gtkw), 1);
+  
+  
+  gtkw = arg_get_value(t, "USE_MAC_ADDR");
+  v = arg_get_value(serv, "use_mac_addr");
+  if(arg_get_type(serv, "use_mac_addr")==ARG_INT)
+  {
+   char * s = emalloc(4);
+   if(v)strncpy(s, "yes", 3);
+   else strncpy(s, "no", 2);
+   
+   arg_set_value(serv, "use_mac_addr", strlen(s), s);
+   arg_set_type(serv, "use_mac_addr", ARG_STRING);
+  }
+  
+  v = arg_get_value(serv, "use_mac_addr");
+  if(v)gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(gtkw), !strcmp(v, "yes"));
+  else gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(gtkw), 0);
+ 
+  
 #ifdef ENABLE_SAVE_KB
 
   /*
@@ -404,10 +476,40 @@ void prefs_dialog_set_defaults(ctrls, preferences)
    if(gtkw)gtk_entry_set_text(GTK_ENTRY(gtkw), v);
   }
  
-  gtkw = arg_get_value(t, "MAX_THREADS");
+  gtkw = arg_get_value(t, "UNSCANNED_CLOSED");
+  v = arg_get_value(serv, "unscanned_closed");
+  if(arg_get_type(serv, "unscanned_closed")==ARG_INT)
+  {
+   
+   char * s;
+   int v;
+   v = (int)arg_get_value(serv, "unscanned_closed");
+   if(v)
+    s = strdup("yes");
+   else
+    s = strdup("no");
+    
+   arg_set_value(serv, "unscanned_closed", strlen(s), s);
+   arg_set_type(serv, "unscanned_closed", ARG_STRING);
+  }
+  v = arg_get_value(serv, "unscanned_closed");
+
+  if(v)gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(gtkw), !strcmp(v, "yes"));
+  else gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(gtkw), 0);
+  
+  
+  
+  gtkw = arg_get_value(t, "MAX_HOSTS");
   if(gtkw)
   {
-   v = arg_get_value(serv, "max_threads");
+   v = arg_get_value(serv, "max_hosts");
+   if(v)gtk_entry_set_text(GTK_ENTRY(gtkw), v);
+  }
+  
+  gtkw = arg_get_value(t, "MAX_CHECKS");
+  if(gtkw)
+  {
+   v = arg_get_value(serv, "max_checks");
    if(v)gtk_entry_set_text(GTK_ENTRY(gtkw), v);
   }
    
@@ -475,6 +577,10 @@ static void prefs_dialog_set_tooltips(ctrls)
  GtkWidget * w;
  tooltips = gtk_tooltips_new();
 
+ t = arg_get_value(ctrls, "PLUGINS");
+ w = arg_get_value(t, "ENABLE_DEPS_AT_RUNTIME");
+ gtk_tooltips_set_tip(tooltips, w, HLP_ENABLE_DEPS_AT_RUNTIME,"");
+ 
  t = arg_get_value(ctrls, "AUTH") ;
 #ifdef USE_AF_INET
  w = arg_get_value(t, "HOSTNAME");
@@ -482,10 +588,7 @@ static void prefs_dialog_set_tooltips(ctrls)
  w = arg_get_value(t, "PORT");
  gtk_tooltips_set_tip(tooltips, w, HLP_AUTH_PORT,"");
 #endif
-#ifdef ENABLE_CRYPTO_LAYER
- w = arg_get_value(t, "CIPHER");
- if(w)gtk_tooltips_set_tip(tooltips, w, HLP_CIPHER_CHOICE,"");
-#endif
+ /*ENABLE_CRYPTO_LAYER*/
  w = arg_get_value(t, "USERNAME");
  gtk_tooltips_set_tip(tooltips, w, HLP_LOGIN_USER,"");
  
@@ -502,15 +605,24 @@ static void prefs_dialog_set_tooltips(ctrls)
  gtk_tooltips_set_tip(tooltips, w, HLP_SCAN_OPT_PING,"");
  w = arg_get_value(t, "OPTIMIZE_TEST");
  gtk_tooltips_set_tip(tooltips, w, HLP_SCAN_OPT_OPTIMIZE, "");
-#ifdef ENABLE_KB
-#warn "No tooltips for detached scan!"
-#endif
+ 
+ w = arg_get_value(t, "SAFE_CHECKS");
+ gtk_tooltips_set_tip(tooltips, w, HLP_SCAN_OPT_SAFE_CHECKS, "");
+ 
+ w = arg_get_value(t, "USE_MAC_ADDR");
+ gtk_tooltips_set_tip(tooltips, w, HLP_SCAN_OPT_USE_MAC_ADDR, "");
+
  w = arg_get_value(t, "REVERSE_LOOKUP");
  gtk_tooltips_set_tip(tooltips, w, HLP_SCAN_OPT_REVERSE_LOOKUP, "");
  w = arg_get_value(t, "PORT_RANGE");
  gtk_tooltips_set_tip(tooltips, w,  HLP_SCAN_OPT_PORT_RANGE, "");
- w = arg_get_value(t, "MAX_THREADS");
- gtk_tooltips_set_tip(tooltips, w, HLP_MISC_MAX_THREADS,"");
+ w = arg_get_value(t, "UNSCANNED_CLOSED");
+ gtk_tooltips_set_tip(tooltips, w, HLP_UNSCANNED_CLOSED, "");
+ 
+ w = arg_get_value(t, "MAX_HOSTS");
+ gtk_tooltips_set_tip(tooltips, w, HLP_MISC_MAX_HOSTS,"");
+ w = arg_get_value(t, "MAX_CHECKS");
+ gtk_tooltips_set_tip(tooltips, w, HLP_MISC_MAX_CHECKS, "");
  w = arg_get_value(t, "CGI_PATH");
  gtk_tooltips_set_tip(tooltips, w, HLP_CGI_PATH, "");
  t = arg_get_value(ctrls, "USER");
@@ -540,6 +652,24 @@ int prefs_dialog_ok(w, ctrls)
    arg_add_value(prefs, "SERVER_PREFS", ARG_ARGLIST, -1, serv);
   }
   
+  
+ t = arg_get_value(ctrls, "PLUGINS");
+ gtkw = arg_get_value(t, "ENABLE_DEPS_AT_RUNTIME");
+ if(gtkw)
+ {
+   char * s;
+   if(GTK_TOGGLE_BUTTON(gtkw)->active)s = estrdup("yes");
+   else s = estrdup("no");
+  
+   if(arg_get_value(serv, "auto_enable_dependencies"))
+   arg_set_value(serv, "auto_enable_dependencies", strlen(s), s);
+  else
+   arg_add_value(serv, "auto_enable_dependencies", ARG_STRING, strlen(s), s);
+  }
+ 
+ 
+  
+  
  /*
   * Host expansion
   */
@@ -580,7 +710,7 @@ int prefs_dialog_ok(w, ctrls)
     
 #endif 
    
-   s = gtk_entry_get_text(GTK_ENTRY(arg_get_value(t, "TARGET")));
+   s = (char*)gtk_entry_get_text(GTK_ENTRY(arg_get_value(t, "TARGET")));
    hostname = target_translate(s); /* if the target is a file, then
    				      translate it */
    
@@ -618,12 +748,14 @@ int prefs_dialog_ok(w, ctrls)
       {
        if((type = (char*)arg_get_value(pref->value, "type")))
        {
-        if(!strcmp(type, PREF_ENTRY))
+        if(!strcmp(type, PREF_ENTRY) ||
+	   !strcmp(type, PREF_FILE) ||
+	   !strcmp(type, PREF_PASSWORD))
          {
 	  GtkWidget * entry = arg_get_value(pref->value, "ENTRY");
 	  if(entry)
 	  {
-	  value = gtk_entry_get_text(GTK_ENTRY(entry));			      
+	  value = (char*)gtk_entry_get_text(GTK_ENTRY(entry));			      
 	  arg_set_value(pref->value, "value", strlen(value), 
 	 	       estrdup(value));
 	  }
@@ -706,6 +838,30 @@ int prefs_dialog_ok(w, ctrls)
     arg_set_value(serv, "optimize_test", strlen(s), s);
    else
     arg_add_value(serv, "optimize_test", ARG_STRING, strlen(s), s);
+    
+    
+   gtkw = arg_get_value(t, "SAFE_CHECKS");
+   s = emalloc(4);
+   if(GTK_TOGGLE_BUTTON(gtkw)->active)strncpy(s, "yes", 3);
+   else strncpy(s, "no", 4);
+   
+   if(arg_get_value(serv, "safe_checks"))
+    arg_set_value(serv, "safe_checks", strlen(s), s);
+   else
+    arg_add_value(serv, "safe_checks", ARG_STRING, strlen(s), s);
+    
+    
+   gtkw = arg_get_value(t, "USE_MAC_ADDR");
+   s = emalloc(4);
+   if(GTK_TOGGLE_BUTTON(gtkw)->active)strncpy(s, "yes", 3);
+   else strncpy(s, "no", 4);
+   
+   if(arg_get_value(serv, "use_mac_addr"))
+    arg_set_value(serv, "use_mac_addr", strlen(s), s);
+   else
+    arg_add_value(serv, "use_mac_addr", ARG_STRING, strlen(s), s);  
+  
+  
   
 #ifdef ENABLE_SAVE_KB
    gtkw = arg_get_value(t, "DETACHED_SCAN");
@@ -723,7 +879,7 @@ int prefs_dialog_ok(w, ctrls)
     arg_add_value(serv, "detached_scan", ARG_STRING, strlen(s), s);  
   
    gtkw = arg_get_value(t, "EMAIL_ADDR");
-   s = gtk_entry_get_text(GTK_ENTRY(gtkw));
+   s = (char*)gtk_entry_get_text(GTK_ENTRY(gtkw));
    arg_set_value(serv, "detached_scan_email_address", strlen(s), s);
    
    
@@ -757,24 +913,47 @@ int prefs_dialog_ok(w, ctrls)
  else
   arg_add_value(serv, "port_range", ARG_STRING, strlen(s), s);
          
- gtkw = arg_get_value(t, "MAX_THREADS");
+	 
+ gtkw = arg_get_value(t, "UNSCANNED_CLOSED");
+ s = emalloc(4);
+ if(GTK_TOGGLE_BUTTON(gtkw)->active)strncpy(s, "yes", 3);
+ else strncpy(s, "no", 4);
+   
+ if(arg_get_value(serv, "unscanned_closed"))
+    arg_set_value(serv, "unscanned_closed", strlen(s), s);
+   else
+    arg_add_value(serv, "unscanned_closed", ARG_STRING, strlen(s), s);
+    
+    	 
+ gtkw = arg_get_value(t, "MAX_HOSTS");
  
  s = emalloc(strlen(gtk_entry_get_text(GTK_ENTRY(gtkw)))+1);
   
  strncpy(s, gtk_entry_get_text(GTK_ENTRY(gtkw)), 
   	strlen(gtk_entry_get_text(GTK_ENTRY(gtkw))));
   
- if(arg_get_value(serv, "max_threads"))
-    arg_set_value(serv, "max_threads", strlen(s), s);
+ if(arg_get_value(serv, "max_hosts"))
+    arg_set_value(serv, "max_hosts", strlen(s), s);
   else
-    arg_add_value(serv, "max_threads", ARG_STRING, strlen(s), s);   
+    arg_add_value(serv, "max_hosts", ARG_STRING, strlen(s), s); 
+    
+    
+ gtkw = arg_get_value(t, "MAX_CHECKS");
+ 
+ s = emalloc(strlen(gtk_entry_get_text(GTK_ENTRY(gtkw)))+1);
+  
+ strncpy(s, gtk_entry_get_text(GTK_ENTRY(gtkw)), 
+  	strlen(gtk_entry_get_text(GTK_ENTRY(gtkw))));
+  
+ if(arg_get_value(serv, "max_checks"))
+    arg_set_value(serv, "max_checks", strlen(s), s);
+  else
+    arg_add_value(serv, "max_checks", ARG_STRING, strlen(s), s);      
  
  
  gtkw = arg_get_value(t, "CGI_PATH");
- s = gtk_entry_get_text(GTK_ENTRY(gtkw));
- s = emalloc(strlen(s)+1);
- strncpy(s, gtk_entry_get_text(GTK_ENTRY(gtkw)), 
-  	strlen(gtk_entry_get_text(GTK_ENTRY(gtkw))));
+ s = estrdup((char*)gtk_entry_get_text(GTK_ENTRY(gtkw)));
+
         
   if(arg_get_value(serv, "cgi_path"))
     arg_set_value(serv, "cgi_path", strlen(s), s);
