@@ -15,6 +15,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
+ * In addition, as a special exception, Renaud Deraison
+ * gives permission to link the code of this program with any
+ * version of the OpenSSL library which is distributed under a
+ * license identical to that listed in the included COPYING.OpenSSL
+ * file, and distribute linked combinations including the two.
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * this file, you may extend this exception to your version of the
+ * file, but you are not obligated to do so.  If you do not wish to
+ * do so, delete this exception statement from your version.
  */
  
 #include <includes.h>
@@ -30,6 +40,8 @@
 #include "../xpm/warning_small.xpm"
 #include "../error_dialog.h"
 #include "prefs_help.h"
+#include "filter.h"
+#include "comm.h"
 
 
 static struct plugin_families * families;
@@ -43,6 +55,45 @@ static void prefs_family_list_callback(GtkWidget * , struct arglist * );
 static GtkWidget * warning_sign(GtkWidget *);
 
 
+
+static int
+file_dialog_hide(GtkWidget * filew, GtkWidget * nul)
+{
+ gtk_widget_hide(filew);
+ gtk_widget_destroy(filew);
+ return 0;
+}
+
+static int
+file_selected(GtkWidget * nul,
+	      GtkWidget * filew)
+{
+ char * fname = (char*)gtk_file_selection_get_filename(GTK_FILE_SELECTION(filew));
+ gtk_widget_hide(filew);
+ 
+ comm_plugin_upload(fname);
+ return 0;
+}	  
+
+
+
+static int hdl_plugin_upload(w, y)
+ GtkWidget * w, * y;
+{
+ GtkWidget * filew = gtk_file_selection_new("Select file");
+ gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filew)->ok_button),
+ 		"clicked", (GtkSignalFunc)file_selected, filew); 
+
+ gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filew)->cancel_button),
+                                 "clicked",
+                                 GTK_SIGNAL_FUNC (file_dialog_hide),
+                                 GTK_OBJECT (filew));			
+ gtk_widget_show(filew);		
+ return 0;		
+}
+
+
+
 static int warning_expl()
 {
  show_info(HLP_WARNING);
@@ -50,7 +101,8 @@ static int warning_expl()
 }
 
 
-static GtkWidget * warning_sign(w)
+static GtkWidget * 
+warning_sign(w)
  GtkWidget * w;
 {
  GdkPixmap * pm;
@@ -80,6 +132,63 @@ static GtkWidget * warning_sign(w)
  return ret;
 }				    
 
+
+static int
+set_filter(w, ctrls)
+ GtkWidget * w;
+ struct arglist * ctrls;
+{
+ struct plugin_filter filter;
+ char * old;
+ struct arglist * plugins = Plugins;
+ GList * dlist;
+
+ ask_filter(&filter);
+ if(!filter.pattern)
+  return 0;
+ 
+
+ if((old = arg_get_value(ctrls, "FILTER")))
+ {
+  arg_set_value(ctrls, "FILTER", sizeof(filter), &filter);
+ }
+ else
+  arg_add_value(ctrls, "FILTER", ARG_STRUCT, sizeof(filter), &filter);
+
+  
+  Filter = filter;
+  memcpy(&Filter, &filter, sizeof(filter));
+  
+  if(plugins)
+   while(plugins->next)
+   {
+    if(filter_plugin(&filter, plugins->value))plug_set_launch(plugins->value, 0);
+    plugins = plugins->next;
+   } 
+  
+  dlist = GTK_LIST(arg_get_value(ctrls,"FAMILIES_LIST"))->children;
+  while(dlist)
+  {
+   char  * name = gtk_object_get_data(GTK_OBJECT(dlist->data), "list_item_data");
+   if(family_empty(name, Plugins))
+    gtk_widget_hide(GTK_WIDGET(dlist->data));
+   else
+    {
+    if(!family_enabled(name, Plugins))
+    {
+      GtkWidget * checkbox = gtk_object_get_data(GTK_OBJECT(dlist->data),
+      							"list_item_checkbox");
+							
+      GTK_TOGGLE_BUTTON(checkbox)->active = 0;							
+    }
+    gtk_widget_show(GTK_WIDGET(dlist->data));
+    }
+   dlist = dlist->next;
+  }
+  prefs_family_list_callback(arg_get_value(ctrls,"FAMILIES_LIST"), ctrls);
+  
+  return 0;
+}
 static int 
 disable_all(w, ctrls)
  GtkWidget * w;
@@ -104,7 +213,8 @@ enable_all(w, ctrls)
  struct arglist * ctrls;
 {
  struct arglist * buttons = arg_get_value(ctrls, "families_buttons");
- while(buttons && buttons->next)
+ if(buttons)
+  while(buttons->next)
  {
   gtk_object_set_data(GTK_OBJECT(buttons->value), "be_lazy", (void*)1);
   family_enable(buttons->name, Plugins, ENABLE_FAMILY);
@@ -123,8 +233,10 @@ enable_all_but_dos(w, ctrls)
  struct arglist * ctrls;
 {
  struct arglist * buttons = arg_get_value(ctrls, "families_buttons");
- while(buttons && buttons->next)
+ if(buttons)
  {
+ while(buttons->next)
+  {
   family_enable(buttons->name, Plugins, ENABLE_FAMILY_BUT_DOS);
   gtk_object_set_data(GTK_OBJECT(buttons->value), "be_lazy", (void*)1);
   if(family_enabled(buttons->name, Plugins))
@@ -133,6 +245,7 @@ enable_all_but_dos(w, ctrls)
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(buttons->value), 0);
   gtk_object_remove_data(GTK_OBJECT(buttons->value), "be_lazy");
   buttons = buttons->next;
+  } 
  }
  pluginset_reload(Plugins, Scanners);
  return 0;
@@ -147,7 +260,8 @@ enable_all_but_dos(w, ctrls)
  * about the plugins of the server
  */
 struct arglist *  
-prefs_dialog_plugins()
+prefs_dialog_plugins(window)
+ GtkWidget * window;
 {
   struct arglist * ctrls = emalloc(sizeof(struct arglist));
   GtkWidget * frame;
@@ -157,7 +271,11 @@ prefs_dialog_plugins()
   GtkWidget * list;
   GtkWidget * vbox, * hbox;
   GtkWidget * button;
- 
+#if GTK_VERSION > 10
+#if GTK_VERSION < 20
+  GtkAccelGroup * accel = gtk_accel_group_new();
+#endif
+#endif
  
   frame = gtk_frame_new("Plugin selection");
   gtk_container_border_width(GTK_CONTAINER(frame), 10);
@@ -168,6 +286,8 @@ prefs_dialog_plugins()
   gtk_container_add(GTK_CONTAINER(frame), w_box);
   
   
+  		     
+  
   families_window = gtk_scrolled_window_new(NULL,NULL);
   gtk_container_border_width(GTK_CONTAINER(families_window), 10);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(families_window),
@@ -179,6 +299,7 @@ prefs_dialog_plugins()
   
   gtk_box_pack_start(GTK_BOX(vbox), families_window, TRUE, TRUE, 0);
   
+  
   hbox = gtk_hbox_new(FALSE, 5);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show(hbox);
@@ -188,9 +309,13 @@ prefs_dialog_plugins()
 			     "clicked",
 			     GTK_SIGNAL_FUNC(enable_all),
 			     ctrls);
+			   
   gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
   gtk_widget_show(button);
   
+ 
+ 
+  
    
   button = gtk_button_new_with_label("Enable all but dangerous plugins");
   gtk_signal_connect(GTK_OBJECT(button),
@@ -209,8 +334,54 @@ prefs_dialog_plugins()
   gtk_widget_show(button);
   
   
+  button = gtk_button_new_with_label("Upload plugin...");
+  gtk_signal_connect(GTK_OBJECT(button),
+			     "clicked",
+			     GTK_SIGNAL_FUNC(hdl_plugin_upload),
+			     ctrls);
+  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+  gtk_widget_show(button);	
   
   
+  hbox = gtk_hbox_new(FALSE, 5);
+  gtk_box_pack_start(GTK_BOX(vbox),
+  		     hbox, 
+		     FALSE, FALSE, 5);
+  gtk_widget_show(hbox);
+  
+  button = gtk_check_button_new_with_label("Enable dependencies at runtime");
+  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+  gtk_widget_show(button);
+  arg_add_value(ctrls, "ENABLE_DEPS_AT_RUNTIME", ARG_PTR, -1, button);
+  
+  
+#if GTK_VERSION < 20
+  button = gtk_button_new_with_label("Filter...");
+#else  
+  button = gtk_button_new_with_mnemonic("Fi_lter...");  
+#endif  
+  gtk_signal_connect(GTK_OBJECT(button),
+  				"clicked",
+				GTK_SIGNAL_FUNC(set_filter),
+				ctrls);
+				
+							     
+#if GTK_VERSION > 10
+#if GTK_VERSION < 20
+  gtk_widget_add_accelerator(GTK_WIDGET(button),
+  			     "clicked",
+			     accel,
+			     'l',
+			     0,
+			     GTK_ACCEL_LOCKED);
+  gtk_window_add_accel_group(GTK_WINDOW(window), accel);
+#endif  
+#endif
+  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+  gtk_widget_show(button);
+
+  		      
+  
   plugins_window = gtk_scrolled_window_new(NULL,NULL);
   gtk_container_border_width(GTK_CONTAINER(plugins_window), 10);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(plugins_window),
@@ -284,8 +455,6 @@ prefs_family_toggle_callback(w, ctrls)
   int enable = GTK_TOGGLE_BUTTON (w)->active;
   GList * dlist = NULL;
   GtkObject * list_item;
-  GtkObject * button;
-  struct arglist * plugin;
   char * name;
   GtkWidget * item;
   int check_manually = 1;
@@ -293,32 +462,16 @@ prefs_family_toggle_callback(w, ctrls)
   if(gtk_object_get_data(GTK_OBJECT(w), "be_lazy"))
    return;
    
-   
+
   item = w->parent->parent;
   list_item = GTK_OBJECT(item);
   name = gtk_object_get_data(list_item,"list_item_data");
   dlist = GTK_LIST(arg_get_value(ctrls, "PLUGINS_LIST"))->children;
-  if(dlist)
-    {
-      list_item = GTK_OBJECT(dlist->data);
-      plugin = gtk_object_get_data(list_item,"plugin");
-      
-      if(plugin && plug_get_family(plugin) && 
-	 !strcmp(plug_get_family(plugin), name))
-	{
-          check_manually = 0;
-	  while(dlist)
-	    {
-	      list_item = GTK_OBJECT(dlist->data);
-	      button = gtk_object_get_data(list_item,"button");
-	      plug_set_launch(plugin, 1);
-	      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), enable);
-	      dlist = dlist->next;
-	    }
-	}
-    }
+
  if(check_manually)family_enable(name, Plugins, enable);
  pluginset_reload(Plugins, Scanners);
+ 
+ prefs_family_list_callback(arg_get_value(ctrls,"FAMILIES_LIST"), ctrls);
 }
 
 static 
@@ -343,10 +496,12 @@ prefs_family_list_callback(widget, ctrls)
   char * cur_family;
   struct arglist * plugs = Plugins;
   GtkTooltips * tooltips;
- 
+  struct plugin_filter * filter = arg_get_value(ctrls, "FILTER");
   
   if(gtk_object_get_data(GTK_OBJECT(widget), "be_lazy"))
    return;
+   
+ 
   
   
   dlist = GTK_LIST(arg_get_value(ctrls, "FAMILIES_LIST"))->selection;
@@ -358,7 +513,7 @@ prefs_family_list_callback(widget, ctrls)
   if(dlist)gtk_list_remove_items(GTK_LIST(arg_get_value(ctrls, "PLUGINS_LIST")),dlist); 
   dlist = NULL;
   tooltips = gtk_tooltips_new();
-  while(plugs && plugs->next)
+  if(plugs)while(plugs->next)
     {
       GtkWidget * item;
       GtkWidget * box;
@@ -366,13 +521,23 @@ prefs_family_list_callback(widget, ctrls)
       GtkWidget * label;
       
       
-      if(plug_get_family(plugs->value) &&
-	 !strcmp(plug_get_family(plugs->value), cur_family))
+      if(arg_get_value(plugs->value, "FAMILY") &&
+	 !strcmp(arg_get_value(plugs->value, "FAMILY"), cur_family))
 	{
 	  char * cat = arg_get_value(plugs->value, "CATEGORY");
-	  int warning = cat? (!strcmp(cat, "denial")):0;
+	  int warning = cat? (!strcmp(cat, "denial") ||
+			      !strcmp(cat, "kill_host") ||
+	  		      !strcmp(cat, "destructive_attack")):0;
 	  GtkWidget * sign = NULL;
 	  	
+	  if(filter)
+	  {
+	   if(filter_plugin(filter, plugs->value))
+	    {
+	     plugs = plugs->next;
+	     continue;
+	    }
+	  }
 	  item = gtk_list_item_new();
 	  
 	  
@@ -449,6 +614,10 @@ fill_plugins_family(ctrls)
  struct arglist * buttons;
  GList * dlist = NULL;
  
+ 
+
+ 
+
  buttons = arg_get_value(ctrls, "families_buttons");
  if(buttons)
  {
@@ -463,17 +632,19 @@ fill_plugins_family(ctrls)
  }
  
 
- 
+ tooltips = gtk_tooltips_new();
  lfamilies = families = family_init();
- while(plugs && plugs->next)
+ if(plugs)
+  while(plugs->next)
     {
       family_add(families, plugs->value);
       plugs = plugs->next;
     }
-  plugs = Plugins;
+ plugs = Plugins;
   
- tooltips = gtk_tooltips_new();
- while(lfamilies && lfamilies->next)
+
+ if(lfamilies)
+  while(lfamilies->next)
    {
      GtkWidget * item;
      GtkWidget * box;
@@ -481,11 +652,13 @@ fill_plugins_family(ctrls)
      GtkWidget * label;
      struct plugin_families * old = arg_get_value(ctrls, "FAMILIES");
      int flag = 0;
-     while(old && old->next && !flag)
-     {
+     if(old)
+      while(old->next && !flag)
+      {
       if(old->name)flag = !strcmp(old->name, lfamilies->name);
       old = old->next;
-     }
+      }
+      
      if(flag){
      	lfamilies = lfamilies->next;
 	continue;
@@ -496,6 +669,8 @@ fill_plugins_family(ctrls)
      
      button = gtk_check_button_new();
      arg_add_value(buttons, lfamilies->name, ARG_PTR, -1, button);
+     gtk_tooltips_set_tip(tooltips, button, lfamilies->name, "");
+     
      gtk_widget_set_usize(button, 15, 15);
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), 
 				 family_enabled(lfamilies->name, Plugins) ? TRUE:FALSE);
@@ -515,6 +690,9 @@ fill_plugins_family(ctrls)
      gtk_widget_show(box);
      gtk_widget_show(item);
      dlist = g_list_append(dlist, item);
+     gtk_object_set_data(GTK_OBJECT(item),
+     			 "list_item_checkbox",
+			 button);
      gtk_object_set_data(GTK_OBJECT(item),
 			 "list_item_data",
 			 lfamilies->name);
