@@ -2,9 +2,8 @@
  * Copyright (C) 1998, 1999, 2000 Renaud Deraison
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2,
+ * as published by the Free Software Foundation
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,6 +14,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
+ * In addition, as a special exception, Renaud Deraison
+ * gives permission to link the code of this program with any
+ * version of the OpenSSL library which is distributed under a
+ * license identical to that listed in the included COPYING.OpenSSL
+ * file, and distribute linked combinations including the two.
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * this file, you may extend this exception to your version of the
+ * file, but you are not obligated to do so.  If you do not wish to
+ * do so, delete this exception statement from your version.
  */  
 
 /* Pluto 26.6.00:
@@ -31,20 +40,21 @@
 #define TOO_MANY_ITEMS "Only the first "MAX_ITEMS_IN_LIST_ASC" entries will be shown.\nExport the report to another format for a complete version"
 
 
-
+#include "gtk-compat.h"
 #include <gtk/gtk.h>
 #include "xstuff.h"
 #include "xpm/yellow.xpm"
 #include "xpm/orange.xpm"
 #include "xpm/red.xpm"
 #include "xpm/white.xpm"
-#include "html_graph_output.h"
+#include "report_ng.h"
 #endif
 
 #include "report.h"
 #include "families.h"
 #include "nsr_output.h"
 #include "html_output.h"
+#include "html_graph_output.h"
 #include "report_utils.h"
 #include "error_dialog.h"
 #include "latex_output.h"
@@ -52,6 +62,7 @@
 #include "xml_output.h"
 #include "globals.h"
 #include "comm.h"
+#include "backend.h"
 
 #include "prefs_dialog/prefs_target.h"
 
@@ -64,6 +75,7 @@
 #define SAVE_MAX SAVE_XML
  
 #ifdef USE_GTK
+
 static void save_report_ask(GtkWidget *, GtkWidget *);
 static void save_report(GtkWidget *, GtkWidget *);
 GtkWidget * report_to_tree(struct arglist *, GtkWidget *);
@@ -355,7 +367,11 @@ No problem has been found at this stage of the test");
   	show_warning("nessusd closed the communication before the end of the test !");
   }
   window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+#if GTK_VERSION > 10
   gtk_window_set_default_size(GTK_WINDOW(window), 640,480);
+#else
+  gtk_widget_set_usize(GTK_WIDGET(window), 640, 480);
+#endif
   gtk_widget_realize(window);
   gtk_signal_connect(GTK_OBJECT(window), "destroy", 
      	GTK_SIGNAL_FUNC(close_window), window);
@@ -725,6 +741,11 @@ int is_there_any_hole(arglist)
  {
   int tmp = 0;
   int tmp2 = 0;
+  if(!arglist->name)
+   {
+    arglist = arglist->next;
+    continue;
+   }
   if(!strcmp(arglist->name, "REPORT"))tmp2 = HOLE_PRESENT;
   else if(!strcmp(arglist->name, "INFO"))tmp2 = WARNING_PRESENT;
   else if(!strcmp(arglist->name, "NOTE"))tmp2 = NOTE_PRESENT;
@@ -740,6 +761,8 @@ int is_there_any_hole(arglist)
 }
 
 #ifdef USE_GTK
+
+
 /*
  *  Main function of the holes reporter
  */
@@ -769,9 +792,8 @@ report_tests(hosts, interrupted_test)
  */
 void open_report(GtkWidget * dontcare, GtkWidget *nsr)
 {
-struct arglist * arguments = emalloc(sizeof(struct arglist));
-file_to_arglist(&arguments, gtk_file_selection_get_filename(GTK_FILE_SELECTION(nsr)) );
-report_tests(arguments, 0);
+int be = backend_import_report(gtk_file_selection_get_filename(GTK_FILE_SELECTION(nsr)));
+if(be >= 0)report_tests_ng(be, 0);
 }
 
 /*
@@ -787,11 +809,15 @@ void open_report_selectfile()
  gtk_file_selection_set_filename (GTK_FILE_SELECTION(nsr), ".");
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (nsr)->ok_button),
                                "clicked", (GtkSignalFunc) open_report,(void *) nsr );
-           
- gtk_signal_connect /*_object*/ (GTK_OBJECT (GTK_FILE_SELECTION
-           (nsr)->cancel_button),
-             "clicked", (GtkSignalFunc)close_window, (void *)nsr);
-           
+
+ gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(nsr)->ok_button),
+   					  "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
+   					  (gpointer) nsr);
+
+ gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(nsr)->cancel_button),
+   					  "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
+   					  (gpointer) nsr);
+
  gtk_widget_show(nsr);
 }
 
