@@ -2,9 +2,8 @@
  * Copyright (C) 1998 Renaud Deraison
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2,
+ * as published by the Free Software Foundation
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,6 +13,17 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * In addition, as a special exception, Renaud Deraison
+ * gives permission to link the code of this program with any
+ * version of the OpenSSL library which is distributed under a
+ * license identical to that listed in the included COPYING.OpenSSL
+ * file, and distribute linked combinations including the two.
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * this file, you may extend this exception to your version of the
+ * file, but you are not obligated to do so.  If you do not wish to
+ * do so, delete this exception statement from your version.
  */
 
 #include <includes.h>
@@ -24,10 +34,11 @@
 #include "nessus.h"
 #include "xstuff.h"
 #include "globals.h"
+#include "sighand.h"
 
-int init_display(int *argc, char * argv[])
+int init_display(int *argc, char *** argv)
 {
- gtk_init(argc, &argv);
+ gtk_init(argc, argv);
  return(0);
 }
 
@@ -44,6 +55,6 @@ int delete_event(GtkWidget * nul, void * data)
 void close_display()
 {
  gtk_main_quit();
- exit(0);
+ nessus_exit(0);
 }
 #endif
