@@ -34,9 +34,12 @@ struct hg_globals {
 	struct hg_host * host_list;    /* List of tested hosts       */
 	struct hg_host * tested;       /* Tested subnets and domains */
 	int		  flags;       /* options		     */
+	char 	       * input;
+	char 	       * marker;
 	};
 		 
 struct hg_globals * hg_init(char *, int);
-char * hg_next_host(struct hg_globals *, struct in_addr *);
+int hg_next_host(struct hg_globals *, struct in_addr *, char *, int);
 void   hg_cleanup  (struct hg_globals *);
+int hg_get_name_from_ip(struct in_addr, char * , int);
 #endif
