@@ -28,7 +28,8 @@ struct attack_atom
  struct attack_atom * next;
 };
 struct arglist * sort_plugins_by_type(struct arglist *);
-int get_max_thread_number(struct arglist *);
+int get_max_hosts_number(struct arglist *);
+int get_max_checks_number(struct arglist *);
 int get_active_plugins_number(struct arglist *);
 void plugins_set_ntp_caps(struct arglist *, ntp_caps*);
 void send_plugin_order(struct arglist *, struct arglist *);
@@ -46,9 +47,14 @@ void attack_atom_remove(struct attack_atom **, char *);
 
 void create_pid_file();
 void delete_pid_file();
-char*temp_file_name(int);
+char*temp_file_name();
 int version_check(char *, char*);
 
+struct arglist * list2arglist(char*);
+int common(struct arglist*, struct arglist*);
 
 int process_alive(pid_t);
+int is_client_present(int);
+int data_left(int);
+int set_linger(int, int);
 #endif
